@@ -32,7 +32,7 @@
 #include	"md_p.h"
 
 int Create(char *mddev, int mdfd,
-	   int chunk, int level, int layout, int size, int raiddisks, int sparedisks,
+	   int chunk, int level, int layout, unsigned long size, int raiddisks, int sparedisks,
 	   int subdevs, mddev_dev_t devlist,
 	   int runstop, int verbose, int force)
 {
@@ -52,7 +52,7 @@ int Create(char *mddev, int mdfd,
 	 * if runstop==run, or raiddisks diskswere used,
 	 * RUN_ARRAY
 	 */
-	int minsize=0, maxsize=0;
+	unsigned long minsize=0, maxsize=0;
 	char *mindisc = NULL;
 	char *maxdisc = NULL;
 	int dnum;
@@ -68,10 +68,10 @@ int Create(char *mddev, int mdfd,
 	
 
 	if (md_get_version(mdfd) < 9000) {
-		fprintf(stderr, Name ": Create requires md driver verison 0.90.0 or later\n");
+		fprintf(stderr, Name ": Create requires md driver version 0.90.0 or later\n");
 		return 1;
 	}
-	if (level == -10) {
+	if (level == UnSet) {
 		fprintf(stderr,
 			Name ": a RAID level is needed to create an array.\n");
 		return 1;
@@ -81,9 +81,19 @@ int Create(char *mddev, int mdfd,
 			Name ": a number of --raid-devices must be given to create an array\n");
 		return 1;
 	}
+	if (raiddisks < 4 && level == 6) {
+		fprintf(stderr,
+			Name ": at least 4 raid-devices needed for level 6\n");
+		return 1;
+	}
+	if (raiddisks > 256 && level == 6) {
+		fprintf(stderr,
+			Name ": no more than 256 raid-devices supported for level 6\n");
+		return 1;
+	}
 	if (raiddisks < 2 && level >= 4) {
 		fprintf(stderr,
-			Name ": atleast 2 raid-devices needed for level 4 or 5\n");
+			Name ": at least 2 raid-devices needed for level 4 or 5\n");
 		return 1;
 	}
 	if (raiddisks+sparedisks > MD_SB_DISKS) {
@@ -102,12 +112,13 @@ int Create(char *mddev, int mdfd,
 	}
 
 	/* now set some defaults */
-	if (layout == -1)
+	if (layout == UnSet)
 		switch(level) {
 		default: /* no layout */
 			layout = 0;
 			break;
 		case 5:
+		case 6:
 			layout = map_name(r5layout, "default");
 			if (verbose)
 				fprintf(stderr,
@@ -115,10 +126,25 @@ int Create(char *mddev, int mdfd,
 			break;
 		}
 
-	if (chunk == 0) {
-		chunk = 64;
-		if (verbose)
-			fprintf(stderr, Name ": chunk size defaults to 64K\n");
+	switch(level) {
+	case 4:
+	case 5:
+	case 6:
+	case 0:
+	case -1: /* linear */
+		if (chunk == 0) {
+			chunk = 64;
+			if (verbose)
+				fprintf(stderr, Name ": chunk size defaults to 64K\n");
+		}
+		break;
+	default: /* raid1, multipath */
+		if (chunk) {
+			chunk = 0;
+			if (verbose)
+				fprintf(stderr, Name ": chunk size ignored for this level\n");
+		}
+		break;
 	}
 
 	/* now look at the subdevs */
@@ -127,7 +153,7 @@ int Create(char *mddev, int mdfd,
 	dnum = 0;
 	for (dv=devlist; dv; dv=dv->next, dnum++) {
 		char *dname = dv->devname;
-		int dsize, freesize;
+		unsigned long dsize, freesize;
 		int fd;
 		if (strcasecmp(dname, "missing")==0) {
 			if (first_missing > dnum)
@@ -153,7 +179,7 @@ int Create(char *mddev, int mdfd,
 			continue;
 		}
 		if (dsize < MD_RESERVED_SECTORS*2) {
-			fprintf(stderr, Name ": %s is too small: %dK\n",
+			fprintf(stderr, Name ": %s is too small: %luK\n",
 				dname, dsize/2);
 			fail = 1;
 			close(fd);
@@ -164,7 +190,7 @@ int Create(char *mddev, int mdfd,
 
 		if (size && freesize < size) {
 			fprintf(stderr, Name ": %s is smaller that given size."
-				" %dK < %dK + superblock\n", dname, freesize, size);
+				" %luK < %luK + superblock\n", dname, freesize, size);
 			fail = 1;
 			close(fd);
 			continue;
@@ -193,10 +219,10 @@ int Create(char *mddev, int mdfd,
 		}
 		size = minsize;
 		if (verbose && level>0)
-			fprintf(stderr, Name ": size set to %dK\n", size);
+			fprintf(stderr, Name ": size set to %luK\n", size);
 	}
 	if (level >= 1 && ((maxsize-size)*100 > maxsize)) {
-		fprintf(stderr, Name ": largest drive (%s) exceed size (%dK) by more than 1%%\n",
+		fprintf(stderr, Name ": largest drive (%s) exceed size (%luK) by more than 1%%\n",
 			maxdisc, size);
 		warn = 1;
 	}
@@ -215,12 +241,19 @@ int Create(char *mddev, int mdfd,
 
 	/* If this is  raid5, we want to configure the last active slot
 	 * as missing, so that a reconstruct happens (faster than re-parity)
+	 * FIX: Can we do this for raid6 as well?
 	 */
-	if (force == 0 && level == 5 && first_missing >= raiddisks) {
-		insert_point = raiddisks-1;
-		sparedisks++;
-		array.active_disks--;
-		missing_disks++;
+	if (force == 0 && first_missing >= raiddisks) {
+		switch ( level ) {
+		case 5:
+			insert_point = raiddisks-1;
+			sparedisks++;
+			array.active_disks--;
+			missing_disks++;
+			break;
+		default:
+			break;
+		}
 	}
 	
 	/* Ok, lets try some ioctls */
@@ -235,8 +268,10 @@ int Create(char *mddev, int mdfd,
 	if (fstat(mdfd, &stb)==0)
 		array.md_minor = MINOR(stb.st_rdev);
 	array.not_persistent = 0;
-	if (level == 5 && (insert_point < raiddisks || first_missing < raiddisks))
-		array.state = 1; /* clean, but one drive will be missing */
+	/*** FIX: Need to do something about RAID-6 here ***/
+	if ( (level == 5 || level == 6) &&
+	     (insert_point < raiddisks || first_missing < raiddisks) )
+		array.state = 1; /* clean, but one+ drive will be missing */
 	else
 		array.state = 0; /* not clean, but no errors */
 
@@ -316,6 +351,7 @@ int Create(char *mddev, int mdfd,
 		if (ioctl(mdfd, RUN_ARRAY, &param)) {
 			fprintf(stderr, Name ": RUN_ARRAY failed: %s\n",
 				strerror(errno));
+			Manage_runstop(mddev, mdfd, -1);
 			return 1;
 		}
 		fprintf(stderr, Name ": array %s started.\n", mddev);
